/*
 
  Code to test wiznet WIZ810MJ module

  See:

     <http://code.rancidbacon.com/LearningAboutArduinoWIZ810MJ>     

  Current features:
  
      * Read register/address values
      
      * Write register/address values
      
      * Configure networking to enable ping (old style)

      * Initial W5100 driver port:
     
          + new-style network configuration
          
          + socket creation/listening/closing
          
          + can't confirm sending

      * Terrible hacked-together code
      
  License:
    
     LGPL
     
     (Although note spi_transfer comes from the Playground originally.)
     
*/

#include <WIZ810MJ.h>

// Define SPI-related pins
#define PIN_DATA_OUT 11 // MOSI (Master Out / Slave In)
#define PIN_DATA_IN  12 // MISO (Master In / Slave Out)
#define PIN_SPI_CLOCK  13 // SCK (Serial Clock) 
#define PIN_SLAVE_SELECT 10 // SS (Slave Select)

#define PIN_RESET 9 // WIZnet module /RESET

#define WIZNET_OPCODE_READ 0x0F
#define WIZNET_OPCODE_WRITE 0xF0

#define DUMMY_DATA 0xFF

SOCKET testSocket;
byte ip[6];


void setup () {
  Serial.begin(9600);
  Serial.println("Setup enter...");

  Serial.print("SPCR: "); Serial.println(SPCR, BIN);

  // Configure SPI
  //   Configure I/O pins
  pinMode(PIN_DATA_OUT, OUTPUT);
  pinMode(PIN_DATA_IN, INPUT);
  pinMode(PIN_SPI_CLOCK, OUTPUT);
  pinMode(PIN_SLAVE_SELECT, OUTPUT);
  
  digitalWrite(PIN_SLAVE_SELECT, HIGH); // Disable slave
  
  //    Configure SPI Control Register (SPCR) (All values initially 0)
  //     Bit  Description
  //       7  SPI Interrupt Enable    -- disable  (SPIE --> 0)
  //       6  SPI Enable              -- enable   (SPE  --> 1)
  //       5  Data Order              -- MSB 1st  (DORD --> 0) (Slave specific)
  //       4  Master/Slave Select     -- master   (MSTR --> 1)
  //       3  Clock Polarity          --          (CPOL --> 0) (Slave specific) ("Mode")
  //       2  Clock Phase             --          (CPHA --> 0) (Slave specific)
  //       1  SPI Clock Rate Select 1 -- }        (SPR1 --> 0) 
  //       0  SPI Clock Rate Select 0 -- } fOSC/4 (SPR0 --> 0) ("Fastest" but see SPI2X in SPSR)
  SPCR = (1<<SPE)| (1<<MSTR);

  Serial.print("SPCR: "); Serial.println(SPCR, BIN);
  
  // Clear previous data and status (TODO: Determine if necessary/better way.)
  // (Based on Playground SPI example.)
  byte dummy;
  dummy = SPSR;
  dummy = SPDR;
  delay(10);
    
  // Serial.println((1 << SPIF), BIN);
  
//  Serial.print("IODIR response: ");
//  iodirVal = getRegister(deviceOpcodeRead, REG_IODIR);
//  Serial.println(iodirVal, BIN);  
  
//  setRegister(deviceOpcodeWrite, REG_IODIR, newVal);
    
  Serial.println("Triggering reset...");
  pinMode(PIN_RESET, OUTPUT);
  
  digitalWrite(PIN_RESET, HIGH); // no reset
  delay(10); // Pretty arbitrary length
  digitalWrite(PIN_RESET, LOW); // reset
  delay(10);
  digitalWrite(PIN_RESET, HIGH); // no reset
  delay(10);
  Serial.println("Reset triggered...");
  
  
  // driver sysinit
  Serial.println("Call sysinit...");
  sysinit(0x55, 0x55);

  // Attempt read
  Serial.println("Read attempt...");
    
  // digitalWrite(PIN_SLAVE_SELECT, LOW); // Enable slave
  
  /*
  byte data;
  
  data = spi_transfer(0x0F); // read
  Serial.print("Response: "); Serial.println(data, HEX);
  
  data = spi_transfer(0x00); // address part1
  Serial.print("Response: "); Serial.println(data, HEX);
  
  data = spi_transfer(0x18); //address part2 //18 should return 0xD0 after reset.
  Serial.print("Response: "); Serial.println(data, HEX);
  
  data = spi_transfer(0xFF); // dummy
  Serial.print("Response: "); Serial.println(data, HEX);
    
  digitalWrite(PIN_SLAVE_SELECT, HIGH); // Disable slave
  */
 
  // RTR Retry Time-value Register 
  Serial.print("Read 0x0017: "); Serial.println(readAddressValue(0x00, 0x17), HEX);  
  Serial.print("Read 0x0018: "); Serial.println(readAddressValue(0x00, 0x18), HEX);  


  Serial.println("Read attempt end...");

    
  Serial.println("Write attempt start...");
  
  Serial.print("Initial read 0x0001: "); Serial.println(readAddressValue(0x00, 0x01), HEX);  
  
  writeAddressValue(0x00, 0x01, 0xC0);

  Serial.print("Post-write read 0x0001: "); Serial.println(readAddressValue(0x00, 0x01), HEX);  
  
  Serial.println("Write attempt end...");

  /*  
  Serial.println("Configure device...");
  
  // default gateway
  writeAddressValue(0x00, 0x01, 0xC0);
  writeAddressValue(0x00, 0x02, 0xA8);
  writeAddressValue(0x00, 0x03, 0x02);
  writeAddressValue(0x00, 0x04, 0x65);
  
  // subnet mask
  writeAddressValue(0x00, 0x05, 0xFF);
  writeAddressValue(0x00, 0x06, 0xFF);
  writeAddressValue(0x00, 0x07, 0xFF);
  writeAddressValue(0x00, 0x08, 0x00);
        
  // source hardware address (MAC?)
  writeAddressValue(0x00, 0x09, 0x00);
  writeAddressValue(0x00, 0x0A, 0xDE);
  writeAddressValue(0x00, 0x0B, 0xAD);
  writeAddressValue(0x00, 0x0C, 0xBE);
  writeAddressValue(0x00, 0x0D, 0xEF);
  writeAddressValue(0x00, 0x0E, 0x00);

  // source ip address
  
  writeAddressValue(0x00, 0x0F, 0xC0);
  writeAddressValue(0x00, 0x10, 0xA8);
  writeAddressValue(0x00, 0x11, 0x02);
  writeAddressValue(0x00, 0x12, 0x69);
  
  
  // Serial.println("SKIP: source ip address");
  
  Serial.println("End configure device...");
  */  

  Serial.println("Test W5100 configuration...");        
  
  byte config_gateway[] = {192,168,2,101};
  byte config_subnet_mask[] = {255,255,255,0};
  byte config_mac_address[] = {0x02,0xDE,0xAD,0xBE,0xEF,0x00};
  byte config_ip_address[] = {192,168,2,105};
   
  setGAR(config_gateway);
  setSUBR(config_subnet_mask);
  setSHAR(config_mac_address); 
  setSIPR(config_ip_address);
   
  Serial.println("End test W5100 configuration...");        
    
        
  Serial.println("Test W5100 driver code...");
  
    //Serial.print(getIR(), HEX);
  Serial.print("GAR0 define: ");
  Serial.println(GAR0, HEX);
    
  Serial.println(getIR(), HEX);
  Serial.println(IINCHIP_READ(RTR0), HEX);
  Serial.println(IINCHIP_READ(RTR0+1), HEX);

  
  getGAR(ip);
  
  Serial.print("Gateway IP read (first digit): ");
  Serial.println(ip[0], DEC);

  Serial.println("End test W5100 driver code...");        

/*
  Serial.println("Test W5100 socket...");
  
  //SOCKET testSocket;

  Serial.print("Create socket result: "); 
  Serial.println(socket(testSocket, Sn_MR_TCP, 5555, 0), DEC);
  
  Serial.print("Listen on socket result: "); 
  Serial.println(listen(testSocket), DEC);

  Serial.println("Waiting for connection..."); 

  while (getSn_SR(testSocket) == SOCK_LISTEN) {
    delay(500);
  }

  Serial.println(getSn_SR(testSocket),HEX);

  getSn_DIPR(testSocket, ip);
  
  Serial.print("Destination IP read (last digit): ");
  Serial.println(ip[3], DEC);
*/
  
  // uint8_t * bytesReceived;
  
  //bytesReceived = (uint8_t *) malloc(8 * sizeof(uint8_t));
  
  //uint16_t ptr = 0;

  //bytesReceived[0] = 'X';
  /*  
  bytesReceived[1] = "X";
  bytesReceived[2] = "X";
  bytesReceived[3] = 0x00;
  */

 /*
  while (getSn_SR(testSocket) == SOCK_ESTABLISHED) {
   while (getSn_RX_RSR(testSocket) > 0) {
    Serial.print("Rx buffer size: ");
    Serial.println(getSn_RX_RSR(testSocket),HEX); // Serial.print(" / ");
    //Serial.print(ptr, HEX);Serial.print(" / ");
    //read_data(testSocket, (uint8_t *) ptr, bytesReceived, 1);
    // Serial.println((char *) bytesReceived);
    bytesReceived = (uint8_t *) RX_BUF;
    
    //Serial.println(bytesReceived[0],HEX);    
    Serial.print("recv result: "); 
    Serial.println(recv(testSocket, bytesReceived, getSn_RX_RSR(testSocket)), DEC);
    Serial.println(bytesReceived[0],HEX);
    Serial.println(bytesReceived[1],HEX);
    Serial.println(bytesReceived[2],HEX);
    
    //ptr++; 
    // Serial.print("Rx buffer size: ");
    //Serial.println(getSn_RX_RSR(testSocket),HEX); // Serial.print(" / ");
    //Serial.println(IINCHIP_READ(getIINCHIP_RxBASE(testSocket)), HEX);
    //Serial.print("Rx base: ");    
    //Serial.println(getIINCHIP_RxBASE(testSocket), HEX);
    delay(1000); 
   }
   delay(500); 
  }  
  
  */
  /*
  uint8_t bytesToSend[] = "XX\nXXXX\0";

  while (getSn_SR(testSocket) == SOCK_ESTABLISHED) {
  Serial.println(send(testSocket, bytesToSend, 3), DEC);

  Serial.print(getSn_IR(testSocket),HEX); Serial.print(" ");  
  Serial.print(getSn_SR(testSocket),HEX); Serial.print(" ");
  Serial.println(getSn_TX_FSR(testSocket),HEX);
  delay(1000);
  }
  */
  /*
  Serial.println("Waiting to receive byte..."); 

  uint8_t bytesReceived[] = "XXXXXXX\0";
  
  //delay(5000);

  Serial.println(bytesReceived[0], HEX);
        
  while (getSn_SR(testSocket) == SOCK_ESTABLISHED) {
  Serial.print("xxxxx "); 
  
  Serial.print("recv result: "); 
  Serial.println(recv(testSocket, bytesReceived, 5), DEC);
  
  Serial.print("recv data: "); 
  //Serial.println((char *)bytesReceived);
  Serial.println(bytesReceived[0], HEX);

  Serial.println(getSn_SR(testSocket),HEX);

  }
  */


  // =================
/*
  while (getSn_SR(testSocket) == SOCK_ESTABLISHED) {
   while (getSn_RX_RSR(testSocket) > 0) {

{
  int len;                                                        
  u_char * data_buf = (u_char*) TX_BUF;
  if ((len = getSn_RX_RSR(testSocket)) > 0) {                       // check Rx data
    if (len > TX_RX_MAX_BUF_SIZE) {
      len = TX_RX_MAX_BUF_SIZE; // if Rx data size is lager than TX_RX_MAX_BUF_SIZE
                                // the data size to read is MAX_BUF_SIZE.
    }
    len = recv(testSocket, data_buf, len);                  // read the received data 
    send(testSocket, data_buf, len);                                // send the received data
  }

}  

   }
  }
*/

  // --------------
  /*
	uint16 src;
//	src = IINCHIP_READ(Sn_RX_RD0(testSocket));
//  Serial.println((uint16_t)src, HEX);


//  Serial.println((uint16_t)src, HEX);
  
    
	uint16 src_mask;
	uint8 * src_ptr;


//  Serial.println(getIINCHIP_RxBASE(testSocket), HEX);



//  Serial.println((long int) src_ptr, HEX);
//  Serial.println((((uint16) src_ptr & 0xFF00)>> 8), HEX);
//  Serial.println((((uint16) src_ptr & 0x00FF)), HEX);


  uint8_t bytesReceived[8];
  
  bytesReceived[0]='a';
  
  //Serial.print("Bytes sent: ");
  //Serial.println(send(testSocket, bytesReceived, 1), HEX);
    
//  Serial.print("Initial read 0x6000: "); Serial.println(readAddressValue(0x60, 0x00), HEX);  

  while (getSn_SR(testSocket) == SOCK_ESTABLISHED) {
   if (getSn_RX_RSR(testSocket) > 0) {
     //Serial.print("Read 0x6000: "); Serial.println(readAddressValue(0x60, 0x00));
     //bytesReceived[0] = 'a';
     //Serial.println(bytesReceived[0], HEX);     
     // wiz_read_buf((uint16)src_ptr, bytesReceived, 3);
     //Serial.println(readAddressValue((((uint16 ) src_ptr) & (uint8) 0xFF00)>>8, (((uint16) src_ptr) & (uint8) 0x00FF)));
     
      src = IINCHIP_READ(Sn_RX_RD0(testSocket));
      src = ((src & 0x00ff) << 8) + IINCHIP_READ(Sn_RX_RD0(testSocket) + 1);
      src_mask = (uint16)src & getIINCHIP_RxMASK(testSocket);
      src_ptr = (uint8 *)(getIINCHIP_RxBASE(testSocket) + src_mask);

     
     //for (int bytesToPrint = getSn_RX_RSR(testSocket); bytesToPrint > 0; bytesToPrint--) {
     //  Serial.print(readAddressValue((((uint16) src_ptr & 0xFF00)>> 8), (((uint16) src_ptr & 0x00FF))));
     //  src_ptr++; // Note: this doesn't account for wrap-around at buffer end.
     //}
     
     wiz_read_buf((uint16)src_ptr, bytesReceived, 3);
     Serial.println(bytesReceived[0]);
     break;  
   }
  }
*/

/*
#define MAX_RX_BUFFER_SIZE 100
  uint8_t bytesReceived[MAX_RX_BUFFER_SIZE]; // NOTE: Actual buffer should be larger.
  
  bytesReceived[0] = 'w';
  bytesReceived[1] = '0';
  bytesReceived[2] = '0';
  bytesReceived[3] = 't';
  bytesReceived[4] = '>';
  bytesReceived[5] = ' ';
  
  Serial.print("send result: "); 
  Serial.println(send(testSocket, bytesReceived, 6), DEC);
  
  
  int dataLength = 0;
  
  while (getSn_SR(testSocket) == SOCK_ESTABLISHED) {
   while (getSn_RX_RSR(testSocket) > 0) {
    dataLength = getSn_RX_RSR(testSocket);
    if (dataLength >= MAX_RX_BUFFER_SIZE) { // TODO: blah, blah...
      dataLength = MAX_RX_BUFFER_SIZE-1;
    }
    Serial.print("dataLength: "); Serial.println(dataLength, HEX);
    Serial.print("recv result: "); 
    Serial.println(recv(testSocket, bytesReceived, dataLength), DEC); // NOTE: Throws away unread portion?
    bytesReceived[dataLength]=0x00;
    Serial.println((char *)bytesReceived);
    Serial.println(bytesReceived[dataLength-1], HEX);
    for (int idx = 0; idx < dataLength; idx++) {
      Serial.print(bytesReceived[idx], HEX); Serial.print(" ");
    }
    Serial.println("");
    
    Serial.print("send result: "); 
    Serial.println(send(testSocket, bytesReceived, dataLength), DEC);
   }}

  disconnect(testSocket);
  close(testSocket);
                                        
  Serial.println("End test W5100 socket...");        
*/                
                                
  Serial.println("Setup exit...");
  
}

byte readAddressValue(byte addressHiByte, byte addressLowByte) {
 // TODO: use a word for the address instead 
  
 byte data = 0x00;
 
 digitalWrite(PIN_SLAVE_SELECT, LOW); // Enable slave
 
 // TODO: Check response values? e.g. 0x00, 0x01, 0x02
 spi_transfer(WIZNET_OPCODE_READ);
 spi_transfer(addressHiByte);
 spi_transfer(addressLowByte);

 data = spi_transfer(DUMMY_DATA);

 digitalWrite(PIN_SLAVE_SELECT, HIGH); // Disable slave
  
 return data;
}


void writeAddressValue(byte addressHiByte, byte addressLowByte, byte targetValue) {
 // TODO: use a word for the address instead 
  
 digitalWrite(PIN_SLAVE_SELECT, LOW); // Enable slave
 
 // TODO: Check response values? e.g. 0x00, 0x01, 0x02
 spi_transfer(WIZNET_OPCODE_WRITE);
 spi_transfer(addressHiByte);
 spi_transfer(addressLowByte);

 spi_transfer(targetValue);

 digitalWrite(PIN_SLAVE_SELECT, HIGH); // Disable slave  
}


/*
byte getRegister(byte targetDeviceOpcode, byte registerAddress) {

  digitalWrite(PIN_SLAVE_SELECT, LOW); // Enable slave
  spi_transfer(targetDeviceOpcode);
  spi_transfer(registerAddress);  // The register we want to read
  
  byte data; // Correct type?
  data = spi_transfer(0xFF); // Transfer dummy byte to get response
  digitalWrite(PIN_SLAVE_SELECT, HIGH); // Disable slave
 
  return data;
} 
*/

/*
void setRegister(byte targetDeviceOpcode, byte registerAddress, byte value) {
  // TODO: Do Better?

  digitalWrite(PIN_SLAVE_SELECT, LOW); // Enable slave
  spi_transfer(targetDeviceOpcode);
  spi_transfer(registerAddress);  // The register we want to write
  spi_transfer(value);
  
  digitalWrite(PIN_SLAVE_SELECT, HIGH); // Disable slave
} 
*/

// From Playground
char spi_transfer(volatile char data)
{
  SPDR = data;                    // Start the transmission
  while (!(SPSR & (1<<SPIF)))     // Wait for the end of the transmission
  {
  };
  return SPDR;                    // return the received byte
}


void loop() {
  // Serial.print(getSn_SR(testSocket),HEX);Serial.print(" ");
  // delay(1000);
  
  Serial.println("Test W5100 socket...");
  
  //SOCKET testSocket;

  Serial.print("Create socket result: "); 
  Serial.println(socket(testSocket, Sn_MR_TCP, 5555, 0), DEC);
  
  Serial.print("Listen on socket result: "); 
  Serial.println(listen(testSocket), DEC);

  Serial.println("Waiting for connection..."); 

  while (getSn_SR(testSocket) == SOCK_LISTEN) {
    delay(500);
  }

  Serial.println(getSn_SR(testSocket),HEX);

  getSn_DIPR(testSocket, ip);
  
  Serial.print("Destination IP read (last digit): ");
  Serial.println(ip[3], DEC);

  
#define MAX_RX_BUFFER_SIZE 100
  uint8_t bytesReceived[MAX_RX_BUFFER_SIZE]; // NOTE: Actual buffer should be larger.
  
  bytesReceived[0] = 'w';
  bytesReceived[1] = '0';
  bytesReceived[2] = '0';
  bytesReceived[3] = 't';
  bytesReceived[4] = '>';
  bytesReceived[5] = ' ';
  
  Serial.print("send result: "); 
  Serial.println(send(testSocket, bytesReceived, 6), DEC);
  
  
  int dataLength = 0;
  
  while (getSn_SR(testSocket) == SOCK_ESTABLISHED) {
   while (getSn_RX_RSR(testSocket) > 0) {
    dataLength = getSn_RX_RSR(testSocket);
    if (dataLength >= MAX_RX_BUFFER_SIZE) { // TODO: blah, blah...
      dataLength = MAX_RX_BUFFER_SIZE-1;
    }
    Serial.print("dataLength: "); Serial.println(dataLength, HEX);
    Serial.print("recv result: "); 
    Serial.println(recv(testSocket, bytesReceived, dataLength), DEC); // NOTE: Throws away unread portion?
    bytesReceived[dataLength]=0x00;
    Serial.println((char *)bytesReceived);
    Serial.println(bytesReceived[dataLength-1], HEX);
    for (int idx = 0; idx < dataLength; idx++) {
      Serial.print(bytesReceived[idx], HEX); Serial.print(" ");
    }
    Serial.println("");
    
    Serial.print("send result: "); 
    Serial.println(send(testSocket, bytesReceived, dataLength), DEC);
   }}

  disconnect(testSocket);
  close(testSocket);
                                        
  Serial.println("End test W5100 socket...");        

  
  
}
